/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：优惠券发放记录表
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.domain;

import java.io.Serializable;

import com.xwd.hospital.server.rest.res.ApiCode;
import lombok.*;
import java.util.List;

import com.baomidou.mybatisplus.annotation.*;

import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.format.annotation.DateTimeFormat;
import com.fasterxml.jackson.annotation.JsonProperty;

import com.xwd.hospital.server.base.BaseDomain;
import com.xwd.hospital.server.enums.CouponTypeEnum;
import com.xwd.hospital.server.domain.CouponInfo;
import com.xwd.hospital.server.domain.UserCouponInfo;
import com.xwd.hospital.server.domain.User;
import com.xwd.hospital.server.domain.UserInfo;

@Data
@Builder(builderMethodName = "newBuilder")
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
@TableName("`pms_coupon_send_record`")
@Schema(name = "CouponSendRecord", description = "优惠券发放记录表")
public class CouponSendRecord extends BaseDomain {
    /**
     * Id
     */
    @TableId(value = "id", type = IdType.AUTO)
    @Schema(description = "Id", nullable = false)
    private Long id;
    /**
     * 优惠券Id
     */
    @TableField(value = "coupon_id")
    @Schema(description = "优惠券Id", nullable = false)
    private Long couponId;
    /**
     * 优惠券Id
     */
    @Schema(description = "优惠券Id")
    @TableField(exist = false)
    private CouponInfo coupon;
    /**
     * 用户优惠券关系id
     */
    @TableField(value = "user_coupon_id")
    @Schema(description = "用户优惠券关系id", nullable = false)
    private Long userCouponId;
    /**
     * 用户优惠券关系id
     */
    @Schema(description = "用户优惠券关系id")
    @TableField(exist = false)
    private UserCouponInfo userCoupon;
    /**
     * 优惠券名称
     */
    @TableField(value = "coupon_name")
    @Schema(description = "优惠券名称", nullable = false)
    private String couponName;
    /**
     * 优惠券类型
     */
    @TableField(value = "coupon_type")
    @Schema(description = "优惠券类型", nullable = false)
    private CouponTypeEnum couponType;
    /**
     * 优惠金额/折扣
     */
    @TableField(value = "coupon_discount")
    @Schema(description = "优惠金额/折扣", nullable = false)
    private java.math.BigDecimal couponDiscount;
    /**
     * 发放人员id
     */
    @TableField(value = "send_id")
    @Schema(description = "发放人员id", nullable = false)
    private Long sendId;
    /**
     * 发放人员id
     */
    @Schema(description = "发放人员id")
    @TableField(exist = false)
    private User send;
    /**
     * 接收人员id
     */
    @TableField(value = "user_info_id")
    @Schema(description = "接收人员id", nullable = false)
    private Long userInfoId;
    /**
     * 接收人员id
     */
    @Schema(description = "接收人员id")
    @TableField(exist = false)
    private UserInfo userInfo;
    /**
     * 操作人Id
     */
    @TableField(value = "editor_id")
    @Schema(description = "操作人Id", nullable = false)
    private Long editorId;
    /**
     * 操作人
     */
    @TableField(value = "editor_name")
    @Schema(description = "操作人", nullable = false)
    private String editorName;
    /**
     * 创建时间
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Schema(description = "创建时间", nullable = false)
    private java.util.Date createTime;
    /**
     * 更新时间
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Schema(description = "更新时间", nullable = false)
    private java.util.Date updateTime;

    /**
     * 设置默认值
     */
    @Override
    public CouponSendRecord setDefault() {
        if (this.getCouponType() == null) {
            this.setCouponType(CouponTypeEnum.FULL_DISCOUNT);
        }
        if (this.getEditorId() == null) {
            this.setEditorId(1L);
        }
        if (this.getEditorName() == null) {
            this.setEditorName("admin");
        }
        if (this.getCreateTime() == null) {
            this.setCreateTime(new java.util.Date());
        }
        if (this.getUpdateTime() == null) {
            this.setUpdateTime(new java.util.Date());
        }
        return this;
    }

    /**
     * 数据验证
     */
    @Override
    public boolean validate(boolean throwException) {
        if (this.getCouponId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "优惠券Id不能为空！");
            }
            return false;
        }
        if (this.getUserCouponId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "用户优惠券关系id不能为空！");
            }
            return false;
        }
        if (this.getCouponName() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "优惠券名称不能为空！");
            }
            return false;
        }
        if (this.getCouponType() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "优惠券类型不能为空！");
            }
            return false;
        }
        if (this.getCouponDiscount() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "优惠金额/折扣不能为空！");
            }
            return false;
        }
        if (this.getSendId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "发放人员id不能为空！");
            }
            return false;
        }
        if (this.getUserInfoId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "接收人员id不能为空！");
            }
            return false;
        }
        if (this.getEditorId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "操作人Id不能为空！");
            }
            return false;
        }
        if (this.getEditorName() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "操作人不能为空！");
            }
            return false;
        }
        return true;
    }

    /**
     * 数据验证
     */
    @Override
    public boolean validate() {
        return this.validate(false);
    }

}
