/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：电话咨询订单记录信息表
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.domain;

import java.io.Serializable;

import com.xwd.hospital.server.rest.res.ApiCode;
import lombok.*;
import java.util.List;

import com.baomidou.mybatisplus.annotation.*;

import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.format.annotation.DateTimeFormat;
import com.fasterxml.jackson.annotation.JsonProperty;

import com.xwd.hospital.server.base.BaseDomain;
import com.xwd.hospital.server.enums.YesNoEnum;
import com.xwd.hospital.server.domain.Order;

@Data
@Builder(builderMethodName = "newBuilder")
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
@TableName("`pms_phone_order_record`")
@Schema(name = "PhoneOrderRecord", description = "电话咨询订单记录信息表")
public class PhoneOrderRecord extends BaseDomain {
    /**
     * Id
     */
    @TableId(value = "id", type = IdType.AUTO)
    @Schema(description = "Id", nullable = false)
    private Long id;
    /**
     * 订单id
     */
    @TableField(value = "order_id")
    @Schema(description = "订单id", nullable = false)
    private Long orderId;
    /**
     * 订单id
     */
    @Schema(description = "订单id")
    @TableField(exist = false)
    private Order order;
    /**
     * 订单号
     */
    @TableField(value = "order_no")
    @Schema(description = "订单号", nullable = false)
    private String orderNo;
    /**
     * 患者被叫号码
     */
    @TableField(value = "patient_phone_no")
    @Schema(description = "患者被叫号码")
    private String patientPhoneNo;
    /**
     * 匿名小号号码
     */
    @TableField(value = "proxy_phone_no")
    @Schema(description = "匿名小号号码")
    private String proxyPhoneNo;
    /**
     * 唯一绑定id
     */
    @TableField(value = "mapping_id")
    @Schema(description = "唯一绑定id")
    private String mappingId;
    /**
     * 是否解绑状态
     */
    @TableField(value = "mapping_state")
    @Schema(description = "是否解绑状态", nullable = false)
    private YesNoEnum mappingState;
    /**
     * 电话咨询开始时间
     */
    @TableField(value = "phone_contact_time")
    @Schema(description = "电话咨询开始时间")
    private String phoneContactTime;
    /**
     * 电话咨询结束时间
     */
    @TableField(value = "phone_release_time")
    @Schema(description = "电话咨询结束时间")
    private String phoneReleaseTime;
    /**
     * 操作人Id
     */
    @TableField(value = "editor_id")
    @Schema(description = "操作人Id", nullable = false)
    private Long editorId;
    /**
     * 操作人
     */
    @TableField(value = "editor_name")
    @Schema(description = "操作人", nullable = false)
    private String editorName;
    /**
     * 创建时间
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Schema(description = "创建时间", nullable = false)
    private java.util.Date createTime;
    /**
     * 更新时间
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Schema(description = "更新时间", nullable = false)
    private java.util.Date updateTime;

    /**
     * 设置默认值
     */
    @Override
    public PhoneOrderRecord setDefault() {
        if (this.getMappingState() == null) {
            this.setMappingState(YesNoEnum.NO);
        }
        if (this.getEditorId() == null) {
            this.setEditorId(1L);
        }
        if (this.getEditorName() == null) {
            this.setEditorName("admin");
        }
        if (this.getCreateTime() == null) {
            this.setCreateTime(new java.util.Date());
        }
        if (this.getUpdateTime() == null) {
            this.setUpdateTime(new java.util.Date());
        }
        return this;
    }

    /**
     * 数据验证
     */
    @Override
    public boolean validate(boolean throwException) {
        if (this.getOrderId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "订单id不能为空！");
            }
            return false;
        }
        if (this.getOrderNo() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "订单号不能为空！");
            }
            return false;
        }
        if (this.getMappingState() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "是否解绑状态不能为空！");
            }
            return false;
        }
        if (this.getEditorId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "操作人Id不能为空！");
            }
            return false;
        }
        if (this.getEditorName() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "操作人不能为空！");
            }
            return false;
        }
        return true;
    }

    /**
     * 数据验证
     */
    @Override
    public boolean validate() {
        return this.validate(false);
    }

}
