/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：转诊记录
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.domain;

import java.io.Serializable;

import com.xwd.hospital.server.rest.res.ApiCode;
import lombok.*;
import java.util.List;

import com.baomidou.mybatisplus.annotation.*;

import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.format.annotation.DateTimeFormat;
import com.fasterxml.jackson.annotation.JsonProperty;

import com.xwd.hospital.server.base.BaseDomain;
import com.xwd.hospital.server.enums.AuditStateEnum;
import com.xwd.hospital.server.domain.PatientInfo;
import com.xwd.hospital.server.domain.HospitalInfo;
import com.xwd.hospital.server.domain.HospitalInfo;
import com.xwd.hospital.server.domain.DoctorInfo;
import com.xwd.hospital.server.domain.DoctorInfo;
import com.xwd.hospital.server.domain.DoctorInfo;
import com.xwd.hospital.server.domain.User;

@Data
@Builder(builderMethodName = "newBuilder")
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
@TableName("`pms_referral_record`")
@Schema(name = "ReferralRecord", description = "转诊记录")
public class ReferralRecord extends BaseDomain {
    /**
     * Id
     */
    @TableId(value = "id", type = IdType.AUTO)
    @Schema(description = "Id", nullable = false)
    private Long id;
    /**
     * 患者Id
     */
    @TableField(value = "patient_id")
    @Schema(description = "患者Id", nullable = false)
    private Long patientId;
    /**
     * 患者Id
     */
    @Schema(description = "患者Id")
    @TableField(exist = false)
    private PatientInfo patient;
    /**
     * 转出医院Id
     */
    @TableField(value = "out_hospital_id")
    @Schema(description = "转出医院Id", nullable = false)
    private Long outHospitalId;
    /**
     * 转出医院Id
     */
    @Schema(description = "转出医院Id")
    @TableField(exist = false)
    private HospitalInfo outHospital;
    /**
     * 转入医院Id
     */
    @TableField(value = "in_hospital_id")
    @Schema(description = "转入医院Id", nullable = false)
    private Long inHospitalId;
    /**
     * 转入医院Id
     */
    @Schema(description = "转入医院Id")
    @TableField(exist = false)
    private HospitalInfo inHospital;
    /**
     * 转出医生Id
     */
    @TableField(value = "out_doctor_id")
    @Schema(description = "转出医生Id", nullable = false)
    private Long outDoctorId;
    /**
     * 转出医生Id
     */
    @Schema(description = "转出医生Id")
    @TableField(exist = false)
    private DoctorInfo outDoctor;
    /**
     * 转入医生Id
     */
    @TableField(value = "in_doctor_id")
    @Schema(description = "转入医生Id", nullable = false)
    private Long inDoctorId;
    /**
     * 转入医生Id
     */
    @Schema(description = "转入医生Id")
    @TableField(exist = false)
    private DoctorInfo inDoctor;
    /**
     * 转诊医生Id
     */
    @TableField(value = "doctor_id")
    @Schema(description = "转诊医生Id", nullable = false)
    private Long doctorId;
    /**
     * 转诊医生Id
     */
    @Schema(description = "转诊医生Id")
    @TableField(exist = false)
    private DoctorInfo doctor;
    /**
     * 审核状态
     */
    @TableField(value = "audit_state")
    @Schema(description = "审核状态", nullable = false)
    private AuditStateEnum auditState;
    /**
     * 审核人Id
     */
    @TableField(value = "audit_id")
    @Schema(description = "审核人Id")
    private Long auditId;
    /**
     * 审核人Id
     */
    @Schema(description = "审核人Id")
    @TableField(exist = false)
    private User audit;
    /**
     * 操作人Id
     */
    @TableField(value = "editor_id")
    @Schema(description = "操作人Id", nullable = false)
    private Long editorId;
    /**
     * 操作人
     */
    @TableField(value = "editor_name")
    @Schema(description = "操作人", nullable = false)
    private String editorName;
    /**
     * 创建时间
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Schema(description = "创建时间", nullable = false)
    private java.util.Date createTime;
    /**
     * 更新时间
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Schema(description = "更新时间", nullable = false)
    private java.util.Date updateTime;

    /**
     * 设置默认值
     */
    @Override
    public ReferralRecord setDefault() {
        if (this.getAuditState() == null) {
            this.setAuditState(AuditStateEnum.TO_AUDIT);
        }
        if (this.getEditorId() == null) {
            this.setEditorId(1L);
        }
        if (this.getEditorName() == null) {
            this.setEditorName("admin");
        }
        if (this.getCreateTime() == null) {
            this.setCreateTime(new java.util.Date());
        }
        if (this.getUpdateTime() == null) {
            this.setUpdateTime(new java.util.Date());
        }
        return this;
    }

    /**
     * 数据验证
     */
    @Override
    public boolean validate(boolean throwException) {
        if (this.getPatientId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "患者Id不能为空！");
            }
            return false;
        }
        if (this.getOutHospitalId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "转出医院Id不能为空！");
            }
            return false;
        }
        if (this.getInHospitalId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "转入医院Id不能为空！");
            }
            return false;
        }
        if (this.getOutDoctorId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "转出医生Id不能为空！");
            }
            return false;
        }
        if (this.getInDoctorId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "转入医生Id不能为空！");
            }
            return false;
        }
        if (this.getDoctorId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "转诊医生Id不能为空！");
            }
            return false;
        }
        if (this.getAuditState() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "审核状态不能为空！");
            }
            return false;
        }
        if (this.getEditorId() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "操作人Id不能为空！");
            }
            return false;
        }
        if (this.getEditorName() == null) {
            if (throwException) {
                throw new ApiCode.ApiException(-1, "操作人不能为空！");
            }
            return false;
        }
        return true;
    }

    /**
     * 数据验证
     */
    @Override
    public boolean validate() {
        return this.validate(false);
    }

}
