package com.xwd.hospital.server.rest;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xwd.hospital.server.annotation.ApiCommon;
import com.xwd.hospital.server.annotation.AuthUser;
import com.xwd.hospital.server.domain.*;
import com.xwd.hospital.server.dto.DepartmentDto;
import com.xwd.hospital.server.dto.ExcelDepartment;
import com.xwd.hospital.server.dto.ExcelUserInfoDto;
import com.xwd.hospital.server.enums.*;
import com.xwd.hospital.server.rest.res.ApiResponse;
import com.xwd.hospital.server.service.*;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@RestController
@Tag(name = "ExcelController", description = "excel导入")
@RequestMapping("/v1/pms/excel")
public class ExcelController {
    @Resource
    private DepartmentInfoService departmentInfoService;
    @Resource
    private UserInfoService userInfoService;
    @Resource
    private PatientInfoService patientInfoService;
    @Resource
    private PatientPrescriptionInfoService prescriptionInfoService;
    @Resource
    private UserCaseInfoService userCaseInfoService;
    @Resource
    private UserAddressInfoService addressInfoService;
    @Resource
    private MedicalReportService medicalReportService;
    /**
     * 导入科室
     */
    @PostMapping("/importDepartments")
    @Operation(summary = "导入科室")
    @ApiCommon
    public ApiResponse<FamilyDoctor> importDepartments(@RequestParam MultipartFile multipartFile) {
        try {
            File file = convertMultipartFileToFile(multipartFile);
            ExcelReader reader = ExcelUtil.getReader(file,"Sheet1");

            List<Map<String,Object>> allData = reader.readAll();
            //将 allData 转换为 ExcelDepartment 的list
            List<ExcelDepartment> excelDepartmentList = allData.stream().map(map -> {
                ExcelDepartment excelDepartment = new ExcelDepartment();
                excelDepartment.setDepartmentName(map.get("科室名称").toString());
                excelDepartment.setDepartmentIntroduce(null== map.get("科室描述")?"":map.get("科室描述").toString());
                //excel中临时序号
                excelDepartment.setExcelId(Long.parseLong(map.get("序号").toString()));
                excelDepartment.setExcelParentId(Long.parseLong(map.get("上级科室序号").toString()));
                return excelDepartment;
            }).collect(Collectors.toList());

            // 使用Collectors.groupingBy进行分组
            Map<Long, List<ExcelDepartment>> excelDepartmentGroups = excelDepartmentList.stream()
                    .collect(Collectors.groupingBy(ExcelDepartment::getExcelParentId));

            // 获取一级部门列表
            List<ExcelDepartment> topLevelDepartments = excelDepartmentGroups.get(0L);

            topLevelDepartments.forEach(excelDepartment -> {
                DepartmentInfo departmentInfo = new DepartmentInfo();
                departmentInfo.setDepartmentName(excelDepartment.getDepartmentName());
                departmentInfo.setDepartmentIntroduce(excelDepartment.getDepartmentIntroduce());
                departmentInfo.setDepartmentLevel(DepartmentLevelEnum.FIRST_LEVEL);
                departmentInfo.setParentId(0L);
                departmentInfoService.save(departmentInfo);
                //插入所有一级科室数据，同时返回记录数据库中的id
                excelDepartment.setId(departmentInfo.getId());
                //根据excelParentId获取当前一级科室下所有的科室
                List<ExcelDepartment> secondeDepartmentList = excelDepartmentGroups.get(excelDepartment.getExcelId());
                secondeDepartmentList.forEach(secondeDepartment -> {
                    DepartmentInfo secondeDepartmentInfo = new DepartmentInfo();
                    secondeDepartmentInfo.setDepartmentName(secondeDepartment.getDepartmentName());
                    secondeDepartmentInfo.setDepartmentIntroduce(secondeDepartment.getDepartmentIntroduce());
                    secondeDepartmentInfo.setDepartmentLevel(DepartmentLevelEnum.SECOND_LEVEL);
                    secondeDepartmentInfo.setParentId(departmentInfo.getId());
                    departmentInfoService.save(secondeDepartmentInfo);
                });
            });

        }catch (IOException e){
            return ApiResponse.fail(-1,"文件导入失败");
        }

        return ApiResponse.ok(null);
    }

    /**
     * 导入患者数据
     */
    @PostMapping("/importUser")
    @Operation(summary = "导入患者数据")
    @ApiCommon
    public ApiResponse<FamilyDoctor> importUser(@RequestParam MultipartFile multipartFile) {
        try {
            File file = convertMultipartFileToFile(multipartFile);
            ExcelReader reader = ExcelUtil.getReader(file,"Sheet1");

            List<Map<String,Object>> allData = reader.readAll();

            //插入user、user_address、处方、medical_report、user_case_info
            //将 allData 转换为 ExcelUserInfoDto 的list
            ArrayList<PatientInfo> patientInfos = new ArrayList<>();
            ArrayList<UserAddressInfo> addressInfos = new ArrayList<>();
            ArrayList<UserCaseInfo> userCaseInfos = new ArrayList<>();
            ArrayList<PatientPrescriptionInfo> prescriptionInfos = new ArrayList<>();
            ArrayList<MedicalReport> medicalReports = new ArrayList<>();
            for(Map<String,Object> map:allData){

                //新增患者信息
                //校验手机号码是否已经存在注册
                UserInfo userInfo = userInfoService.getOne(Wrappers.<UserInfo>query().eq("phone_number", map.get("手机号").toString())
                        .eq("delete_status", YesNoEnum.NO));
                if(null == userInfo){
                    userInfo = new UserInfo();
                    userInfo.setUserId(0L);//关联sys_user表的id，暂时记为0
                    userInfo.setName(map.get("姓名").toString());
                    userInfo.setSex("男".equals(map.get("性别").toString())? SexTypeEnum.MALE:SexTypeEnum.FEMALE);
                    String idNo = map.get("证件号").toString();

                    if(!IdcardUtil.isValidCard(idNo)){
                        continue;
                    };

                    userInfo.setIdNo(idNo);



                    userInfo.setPhoneNumber(map.get("手机号").toString());
                    DateTime birthday = DateUtil.parse(IdcardUtil.getBirthByIdCard(userInfo.getIdNo()));
                    userInfo.setBornDate(DateUtil.format(birthday,"yyyy-MM-dd"));
                    userInfo.setPersonalInfoFilled(YesNoEnum.NO);
                    userInfoService.save(userInfo);
                }
                //就诊人patient
                PatientInfo patientInfo = patientInfoService.getOne(Wrappers.<PatientInfo>query().eq("user_info_id", userInfo.getId())
                        .eq("relation", PatientRelationEnum.SELF).eq("id_no",userInfo.getIdNo()).eq("delete_status", YesNoEnum.NO));
                if(null == patientInfo){
                    //同时创建自身的就诊人
                    patientInfo = new PatientInfo();
                    patientInfo.setUserInfoId(userInfo.getId());
                    patientInfo.setRelation(PatientRelationEnum.SELF);
                    patientInfo.setPatientName(userInfo.getName());
                    patientInfo.setSex(userInfo.getSex());
                    patientInfo.setIdNo(userInfo.getIdNo());
                    patientInfo.setPhoneNumber(userInfo.getPhoneNumber());
                    patientInfo.setBornDate(userInfo.getBornDate());
                    patientInfoService.save(patientInfo);
                }

                //患者地址
                String province = null == map.get("省") ? "" : map.get("省").toString();
                String city = null == map.get("省") ? "" : map.get("省").toString();
                String district = null == map.get("省") ? "" : map.get("省").toString();
                if(!StringUtils.isEmpty(province)
                        && !StringUtils.isEmpty(city)
                        && !StringUtils.isEmpty(district)){
                    UserAddressInfo userAddressInfo = new UserAddressInfo();
                    userAddressInfo.setUserInfoId(userInfo.getId());
                    userAddressInfo.setName(userInfo.getName());
                    userAddressInfo.setPhoneNumber(userInfo.getPhoneNumber());
                    userAddressInfo.setProvince(province);
                    userAddressInfo.setCity(city);
                    userAddressInfo.setDistrict(district);
                    userAddressInfo.setDetailAddress(null== map.get("详细地址")?"":map.get("详细地址").toString());
                    addressInfos.add(userAddressInfo);
                }


                //病例信息
                String anamnesis = null == map.get("既往病史") ? "" : map.get("既往病史").toString();
                String history = null == map.get("药物过敏史") ? "" : map.get("药物过敏史").toString();
                if(!StringUtils.isEmpty(anamnesis)
                        || !StringUtils.isEmpty(history)){
                    UserCaseInfo userCaseInfo = new UserCaseInfo();
                    userCaseInfo.setUserInfoId(userInfo.getId());
                    userCaseInfo.setName(userInfo.getName());
                    userCaseInfo.setPhoneNumber(userInfo.getPhoneNumber());
                    userCaseInfo.setAnamnesis(anamnesis);
                    userCaseInfo.setDrugAllergyHistory(history);
                    userCaseInfos.add(userCaseInfo);
                }

                //处方信息
                String hospital = null == map.get("处方医院") ? "" : map.get("处方医院").toString();
                String department = null == map.get("处方科室") ? "" : map.get("处方科室").toString();
                String time = null == map.get("处方时间") ? "" : map.get("处方时间").toString();
                String result = null == map.get("处方结果") ? "" : map.get("处方结果").toString();
                if(!StringUtils.isEmpty(hospital)
                        && !StringUtils.isEmpty(department)
                        && !StringUtils.isEmpty(time)
                        && !StringUtils.isEmpty(result)){
                    PatientPrescriptionInfo patientPrescriptionInfo = new PatientPrescriptionInfo();
                    patientPrescriptionInfo.setPatientId(patientInfo.getId());
                    patientPrescriptionInfo.setHospitalName(hospital);
                    patientPrescriptionInfo.setDepartmentName(department);
                    patientPrescriptionInfo.setPrescriptionTime(time);
                    patientPrescriptionInfo.setPrescriptionIntroduce(result);
                    prescriptionInfos.add(patientPrescriptionInfo);
                }

                //报告信息
                MedicalReport medicalReport = new MedicalReport();
                medicalReport.setAge(Integer.valueOf(null == map.get("年龄") ? "" :map.get("年龄").toString()));
                medicalReport.setPatientName(null == map.get("姓名") ? "" :map.get("姓名").toString());
                medicalReport.setReportDoctor(null == map.get("报告医生") ? "" :map.get("报告医生").toString());
                medicalReport.setReportHospital(null == map.get("报告医院") ? "" :map.get("报告医院").toString());
                medicalReport.setPatientIdNo(null == map.get("姓名") ? "" :map.get("姓名").toString());
                medicalReport.setSex("男".equals(null == map.get("性别") ? SexTypeEnum.MALE :map.get("性别").toString())? SexTypeEnum.MALE :SexTypeEnum.FEMALE);
                medicalReport.setReportItem(null == map.get("报告项目") ? "" :map.get("报告项目").toString());
                medicalReport.setReportDeclare(null == map.get("报告描述") ? "" :map.get("报告描述").toString());
                medicalReport.setReportTime(null == map.get("报告时间") ? new Date() :DateUtil.parse(map.get("报告时间").toString()));
                medicalReport.setReportType(ReportTypeEnum.CT);
                medicalReport.setDoctorAsks(null == map.get("医生医嘱") ? "" :map.get("医生医嘱").toString());
                medicalReports.add(medicalReport);
            }

            patientInfoService.saveBatch(patientInfos);
            addressInfoService.saveBatch(addressInfos);
            userCaseInfoService.saveBatch(userCaseInfos);
            prescriptionInfoService.saveBatch(prescriptionInfos);
            medicalReportService.saveBatch(medicalReports);

        }catch (Exception e){
            return ApiResponse.fail(-1,"字段缺失或异常，文件导入失败");
        }

        return ApiResponse.ok(null);
    }


    public File convertMultipartFileToFile(MultipartFile multipartFile) throws IOException {
        File file = new File(multipartFile.getOriginalFilename());
        FileUtils.writeByteArrayToFile(file, multipartFile.getBytes());
        return file;
    }

}
