/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：家庭医生
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.rest;

import java.util.List;
import java.util.Arrays;

import cn.dev33.satoken.stp.SaLoginModel;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.IdcardUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.xwd.hospital.server.annotation.ApiCommon;
import com.xwd.hospital.server.annotation.AuthUser;
import com.xwd.hospital.server.base.BaseController;
import com.xwd.hospital.server.domain.User;
import com.xwd.hospital.server.rest.res.ApiCode;
import com.xwd.hospital.server.domain.FamilyDoctor;
import com.xwd.hospital.server.rest.req.FamilyDoctorParam;
import com.xwd.hospital.server.rest.res.ApiResponse;
import com.xwd.hospital.server.service.FamilyDoctorService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@Tag(name = "FamilyDoctorController", description = "家庭医生管理")
@RequestMapping("/v1/pms/family-doctor")
public class FamilyDoctorController extends BaseController<FamilyDoctorService, FamilyDoctor, FamilyDoctorParam> {

    @Resource
    public void setService(FamilyDoctorService service) {
        this.service = service;
    }

    /**
     * 新增
     *
     * @param entity 要新增的对象
     * @return 新增的对象
     */
    @PostMapping("/")
    @Operation(summary = "新增")
    @Override
    public ApiResponse<FamilyDoctor> add(@RequestBody FamilyDoctor entity, @AuthUser User user) {

        if(StringUtils.isEmpty(entity.getIdNo())){
            return ApiResponse.fail(ApiCode.FAILED.getCode(),"身份证号码验证不正确");
        }
        if(!IdcardUtil.isValidCard(entity.getIdNo())){
            return ApiResponse.fail(ApiCode.FAILED.getCode(),"身份证号码验证不正确");
        }

        entity.setEditorId(user.getId());
        entity.setEditorName(user.getUsername());
        entity.setDefault().validate(true);
        this.service.save(entity);
        return ApiResponse.ok(this.service.getById(entity.getId()));
    }

    /**
     * 批量新增
     *
     * @param entityList 要新增的对象
     * @return boolean 成功或失败
     */
    @PostMapping("/batch-save")
    @Operation(summary = "批量新增")
    @Override
    public ApiResponse<Boolean> batchSave(@RequestBody List<FamilyDoctor> entityList, @AuthUser User user) {
        entityList.forEach(entity -> {
            entity.setEditorId(user.getId());
            entity.setEditorName(user.getUsername());
            entity.setDefault().validate(true);
        });
        return ApiResponse.ok(this.service.saveBatch(entityList));
    }

    /**
     * 医生确认
     */
    @PostMapping("/confirm/{id}")
    @Operation(summary = "医生确认")
    public ApiResponse confirm(@PathVariable("id") Long id, @AuthUser User user) {
        return ApiResponse.ok(this.service.confirm(id,user));
    }

    /**
     * 医生拒绝
     */
    @PostMapping("/reject/{id}")
    @Operation(summary = "医生拒绝")
    public ApiResponse reject(@PathVariable("id") Long id,@RequestBody String rejectReason, @AuthUser User user) {
        return ApiResponse.ok(this.service.reject(id, rejectReason,user));
    }

    /**
     * 审核通过
     */
    @PostMapping("/auditPass/{id}")
    @Operation(summary = "审核通过")
    public ApiResponse auditPass(@PathVariable("id") Long id, @AuthUser User user) {
        return ApiResponse.ok(this.service.auditPass(id,user));
    }

    /**
     * 审核失败
     */
    @PostMapping("/auditFail/{id}")
    @Operation(summary = "审核失败")
    public ApiResponse auditFail(@PathVariable("id") Long id,@RequestBody String rejectReason, @AuthUser User user) {
        return ApiResponse.ok(this.service.auditFail(id, rejectReason,user));
    }


}
