/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：用户病例关系表
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.rest.req;

import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.v3.oas.annotations.Parameter;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import com.xwd.hospital.server.base.BaseParam;
import com.xwd.hospital.server.domain.UserCaseInfo;

@Data
@EqualsAndHashCode(callSuper = true)
@Builder(builderMethodName = "newBuilder")
@NoArgsConstructor
@AllArgsConstructor
public class UserCaseInfoParam extends BaseParam<UserCaseInfo> {
    /**
     * Id
     */
    @Parameter(description = "Id")
    private Long id;
    /**
     * Id IN值List
     */
    @Parameter(description = "Id IN值List")
    private List<Long> idList;
    /**
     * 用户Id
     */
    @Parameter(description = "用户Id")
    private Long userInfoId;
    /**
     * 姓名
     */
    @Parameter(description = "姓名")
    private String name;
    /**
     * 手机号码
     */
    @Parameter(description = "手机号码")
    private String phoneNumber;
    /**
     * 既往病史
     */
    @Parameter(description = "既往病史")
    private String anamnesis;
    /**
     * 药物过敏史
     */
    @Parameter(description = "药物过敏史")
    private String drugAllergyHistory;
    /**
     * 操作人Id
     */
    @Parameter(description = "操作人Id")
    private Long editorId;
    /**
     * 操作人
     */
    @Parameter(description = "操作人")
    private String editorName;
    /**
     * 创建时间
     */
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Parameter(description = "创建时间")
    private java.util.Date createTime;
    /**
     * 创建时间 下限值(大于等于)
     */
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Parameter(description = "创建时间 下限值(大于等于)")
    private java.util.Date createTimeFrom;
    /**
     * 创建时间 上限值(小于)
     */
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Parameter(description = "创建时间 上限值(小于)")
    private java.util.Date createTimeTo;
    /**
     * 更新时间
     */
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Parameter(description = "更新时间")
    private java.util.Date updateTime;
    /**
     * 更新时间 下限值(大于等于)
     */
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Parameter(description = "更新时间 下限值(大于等于)")
    private java.util.Date updateTimeFrom;
    /**
     * 更新时间 上限值(小于)
     */
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @Parameter(description = "更新时间 上限值(小于)")
    private java.util.Date updateTimeTo;


    /**
     * 将查询参数转化为查询Wrapper
     */
    @Override
    public QueryWrapper<UserCaseInfo> toQueryWrapper() {
        return this.toQueryWrapper("");
    }

    /**
     * 将查询参数转化为查询Wrapper
     */
    @Override
    public QueryWrapper<UserCaseInfo> toQueryWrapper(String columnPrefix) {
        QueryWrapper<UserCaseInfo> wrapper = Wrappers.<UserCaseInfo>query();
        wrapper.eq(this.getId() != null, columnPrefix + "id", this.getId());
        wrapper.in(this.getIdList() != null && this.getIdList().size() > 0, columnPrefix + "id", this.getIdList());
        wrapper.eq(this.getUserInfoId() != null, columnPrefix + "user_info_id", this.getUserInfoId());
        if (this.getName() != null) {
            if (this.getName().startsWith("%") && this.getName().endsWith("%")) {
                wrapper.like(columnPrefix + "name", this.getName().substring(1, this.getName().length() - 1));
            } else if (this.getName().startsWith("%") && !this.getName().endsWith("%")) {
                wrapper.likeLeft(columnPrefix + "name", this.getName().substring(1));
            } else if (this.getName().endsWith("%")) {
                wrapper.likeRight(columnPrefix + "name", this.getName().substring(0, this.getName().length() - 1));
            } else {
                wrapper.eq(columnPrefix + "name", this.getName());
            }
        }
        if (this.getPhoneNumber() != null) {
            if (this.getPhoneNumber().startsWith("%") && this.getPhoneNumber().endsWith("%")) {
                wrapper.like(columnPrefix + "phone_number", this.getPhoneNumber().substring(1, this.getPhoneNumber().length() - 1));
            } else if (this.getPhoneNumber().startsWith("%") && !this.getPhoneNumber().endsWith("%")) {
                wrapper.likeLeft(columnPrefix + "phone_number", this.getPhoneNumber().substring(1));
            } else if (this.getPhoneNumber().endsWith("%")) {
                wrapper.likeRight(columnPrefix + "phone_number", this.getPhoneNumber().substring(0, this.getPhoneNumber().length() - 1));
            } else {
                wrapper.eq(columnPrefix + "phone_number", this.getPhoneNumber());
            }
        }
        if (this.getAnamnesis() != null) {
            if (this.getAnamnesis().startsWith("%") && this.getAnamnesis().endsWith("%")) {
                wrapper.like(columnPrefix + "anamnesis", this.getAnamnesis().substring(1, this.getAnamnesis().length() - 1));
            } else if (this.getAnamnesis().startsWith("%") && !this.getAnamnesis().endsWith("%")) {
                wrapper.likeLeft(columnPrefix + "anamnesis", this.getAnamnesis().substring(1));
            } else if (this.getAnamnesis().endsWith("%")) {
                wrapper.likeRight(columnPrefix + "anamnesis", this.getAnamnesis().substring(0, this.getAnamnesis().length() - 1));
            } else {
                wrapper.eq(columnPrefix + "anamnesis", this.getAnamnesis());
            }
        }
        if (this.getDrugAllergyHistory() != null) {
            if (this.getDrugAllergyHistory().startsWith("%") && this.getDrugAllergyHistory().endsWith("%")) {
                wrapper.like(columnPrefix + "drug_allergy_history", this.getDrugAllergyHistory().substring(1, this.getDrugAllergyHistory().length() - 1));
            } else if (this.getDrugAllergyHistory().startsWith("%") && !this.getDrugAllergyHistory().endsWith("%")) {
                wrapper.likeLeft(columnPrefix + "drug_allergy_history", this.getDrugAllergyHistory().substring(1));
            } else if (this.getDrugAllergyHistory().endsWith("%")) {
                wrapper.likeRight(columnPrefix + "drug_allergy_history", this.getDrugAllergyHistory().substring(0, this.getDrugAllergyHistory().length() - 1));
            } else {
                wrapper.eq(columnPrefix + "drug_allergy_history", this.getDrugAllergyHistory());
            }
        }
        wrapper.eq(this.getEditorId() != null, columnPrefix + "editor_id", this.getEditorId());
        if (this.getEditorName() != null) {
            if (this.getEditorName().startsWith("%") && this.getEditorName().endsWith("%")) {
                wrapper.like(columnPrefix + "editor_name", this.getEditorName().substring(1, this.getEditorName().length() - 1));
            } else if (this.getEditorName().startsWith("%") && !this.getEditorName().endsWith("%")) {
                wrapper.likeLeft(columnPrefix + "editor_name", this.getEditorName().substring(1));
            } else if (this.getEditorName().endsWith("%")) {
                wrapper.likeRight(columnPrefix + "editor_name", this.getEditorName().substring(0, this.getEditorName().length() - 1));
            } else {
                wrapper.eq(columnPrefix + "editor_name", this.getEditorName());
            }
        }
        wrapper.eq(this.getCreateTime() != null, columnPrefix + "create_time", this.getCreateTime());
        wrapper.ge(this.getCreateTimeFrom() != null, columnPrefix + "create_time", this.getCreateTimeFrom());
        wrapper.lt(this.getCreateTimeTo() != null, columnPrefix + "create_time", this.getCreateTimeTo());
        wrapper.eq(this.getUpdateTime() != null, columnPrefix + "update_time", this.getUpdateTime());
        wrapper.ge(this.getUpdateTimeFrom() != null, columnPrefix + "update_time", this.getUpdateTimeFrom());
        wrapper.lt(this.getUpdateTimeTo() != null, columnPrefix + "update_time", this.getUpdateTimeTo());
        return wrapper;
    }

}
