/**
 * 项目：互联网医疗
 * 模型分组：服务管理
 * 模型名称：转诊记录
 * @Author: xiongwei
 * @Date: 2023-09-05 09:42:00
 */

package com.xwd.hospital.server.service.impl;

import java.io.Serializable;
import java.util.List;
import java.util.ArrayList;
import java.util.Collection;
import java.util.stream.Collectors;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xwd.hospital.server.domain.PatientInfo;
import com.xwd.hospital.server.enums.AuditStateEnum;
import com.xwd.hospital.server.enums.YesNoEnum;
import com.xwd.hospital.server.rest.res.ApiResponse;
import com.xwd.hospital.server.service.PatientInfoService;
import jakarta.annotation.Resource;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.xwd.hospital.server.service.ReferralRecordService;
import com.xwd.hospital.server.repository.ReferralRecordMapper;
import com.xwd.hospital.server.repository.base.ReferralRecordBaseMapper;
import com.xwd.hospital.server.domain.ReferralRecord;
import com.xwd.hospital.server.rest.req.ReferralRecordParam;

@Service
public class ReferralRecordServiceImpl extends ServiceImpl<ReferralRecordMapper, ReferralRecord> implements ReferralRecordService {

    @Resource
    private PatientInfoService patientInfoService;

    @Override
    public int updateAllFieldsById(ReferralRecord entity) {
        return this.getBaseMapper().updateAllFieldsById(entity);
    }

    /**
     * 批量插入
     *
     * @param entityList ignore
     * @param batchSize  ignore
     * @return ignore
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveBatch(Collection<ReferralRecord> entityList, int batchSize) {
        String sqlStatement = SqlHelper.getSqlStatement(ReferralRecordBaseMapper.class, SqlMethod.INSERT_ONE);
        return executeBatch(entityList, batchSize, (sqlSession, entity) -> sqlSession.insert(sqlStatement, entity));
    }

    @Override
    public boolean audit(ReferralRecord entity) {
        Object userId = StpUtil.getExtra("userId");
        entity.setAuditId(Long.valueOf(userId.toString()));
        return this.saveOrUpdate(entity);
    }

    @Override
    public IPage<ReferralRecord> listMyRecord(Page<ReferralRecord> page, Long userInfoId) {
        List<PatientInfo> patientInfoList = patientInfoService.lambdaQuery().eq(PatientInfo::getUserInfoId, userInfoId).eq(PatientInfo::getDeleteStatus, YesNoEnum.NO).list();
        List<Long> patientIdList = patientInfoList.stream().map(PatientInfo::getId).collect(Collectors.toList());
        if(null == patientIdList || patientIdList.size() == 0){
            return null;
        }
        QueryWrapper<ReferralRecord> wrapper  = new QueryWrapper<ReferralRecord>().in("patient_id", patientIdList)
                .orderByDesc("create_time");

        return this.page(page, wrapper);
    }
}
